/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package membrane.interceptor;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.julia.ComponentInterface;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.Interceptor;
import org.objectweb.fractal.juliak.control.interceptor.InterceptorController;
import org.objectweb.fractal.util.Fractal;

/**
 * This class illustrates the usage of the interceptor controller for
 * dynamically adding and removing an interceptor.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class Main {
    
    public static void main(String[] args) throws Exception {
        
        Component boot = Fractal.getBootstrapComponent();
        TypeFactory tf = Fractal.getTypeFactory(boot);
        GenericFactory gf = Fractal.getGenericFactory(boot);
        
        /*
         * Create the component type.
         */
        ComponentType type =
            tf.createFcType(
                new InterfaceType[]{
                    tf.createFcItfType(
                        "r", Runnable.class.getName(), false, false, false)
                }
            );
        
        /*
         * Create a primitive component, start it and invoke it.
         */
        Component c = gf.newFcInstance(type, "primitive", C.class.getName());
        Fractal.getLifeCycleController(c).startFc();
        ComponentInterface itf = (ComponentInterface) c.getFcInterface("r");
        ((Runnable)itf).run();
        
        /*
         * Adding an interceptor.
         */
        System.err.println("<< Adding an interceptor");
        InterceptorController ic = (InterceptorController)
            c.getFcInterface("/interceptor-controller");
        Interceptor myInterceptor = new MyInterceptor();
        ic.addFcInterceptor(itf, myInterceptor);
        
        /*
         * Re-invoking the component.
         */
        ((Runnable)itf).run();

        /*
         * Removing the interceptor.
         */
        System.err.println("<< Removing the interceptor");
        ic.removeFcInterceptor(itf, myInterceptor);

        /*
         * Re-invoking the component.
         */
        ((Runnable)itf).run();
    }
}

class MyInterceptor implements Runnable, Interceptor {

    public void run() {
        System.err.println("   [[ In the interceptor ]]");
        delegate.run();
    }

    public Object getFcItfDelegate() {
        return delegate;
    }

    public void setFcItfDelegate(Object delegate) {
        this.delegate = (Runnable) delegate;
    }
    
    private Runnable delegate;

    public void initFcController(InitializationContext ic) throws InstantiationException {
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch( CloneNotSupportedException cnse ) {
        }
        return null;
    }
    
}