/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.asm;

import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.asm.AttributeControllerClassGenerator;
import org.objectweb.fractal.julia.loader.Tree;


/**
 * This class generates cloneable attribute controller implementations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class CloneableAttributeControllerClassGenerator
    extends AbstractClassGenerator {
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    protected String getClassNameSuffix( InterfaceType it ) {
        return "AC";
    }

    /**
     * Return true if the mode (IN, OUT or IN_OUT) of the class generator matchs
     * the mode (client or server) of the given interface type.
     */
    public boolean match( InterfaceType it ) {
        return true;
    }

    /**
     * Generate the bytecode of a class implementing the given Fractal interface
     * type.
     * 
     * @param targetClassname    the name of the class to generate
     * @param it  the corresponding interface type
     * @param ic  the context for initializing the generated class
     * @return    the bytecode of the generated class
     */
    public byte[] generate(
            String targetClassname, InterfaceType it,
            InitializationContext ic )
    throws InstantiationException {
        
        AttributeControllerClassGenerator accg =
            new AttributeControllerClassGenerator();
        String signature = it.getFcItfSignature();
        Tree args =
            new Tree(
                new Tree[]{
                    new Tree(AttributeControllerClassGenerator.class.getName()),
                    new Tree(
                        new Tree[]{
                            new Tree(signature)
                        }
                    )
                }
            );
        
        byte[] bytecode = null;
        try {
            bytecode =
                accg.generateClass( targetClassname, args, getLoader(), getClassLoader() );
        }
        catch( Exception cge ) {
            /*
             * Actual thrown exception is
             * org.objectweb.julia.asm.ClassGenerationException. Catch Exception
             * to avoid introducing a dependency towards julia-asm.jar. It seems
             * that caught exceptions introduce a dependency as soon as the
             * class even if the code is not reached.
             */
            final String msg = "ClassGenerationException: "+cge.getMessage();
            throw new InstantiationException(msg);
        }
        
        return bytecode;
    }
}
