/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.factory;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.fractal.juliak.Juliak;
import org.objectweb.fractal.juliak.platform.Platform;
import org.objectweb.fractal.juliak.platform.PlatformItf;

/**
 * Repository for classes dynamically loaded by Juliak.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class ClasseDefinitions {
    
    /**
     * Return the singleton instance of this class.
     */
    public static ClasseDefinitions get() {
        return singleton;
    }
    private static ClasseDefinitions singleton = new ClasseDefinitions();
    private ClasseDefinitions() {}

    /** The map of classes. */
    private Map classes = new HashMap();

    
    /**
     * Return true if the repository contains the specified class name.
     */
    public boolean containsClass(String targetClassname) {
        boolean b = classes.containsKey(targetClassname);
        return b;
    }
    
    /**
     * Retrieve a class from the repository.
     */
    public ClassDefinition getClass(String targetClassname) {
        Object o = classes.get(targetClassname);
        return (ClassDefinition) o;
    }
    
    /**
     * Register a class in the repository.
     */
    public ClassDefinition registerClass(String targetClassname, byte[] bytecode) {
        
        /*
         * Load the class in the VM.
         */
        PlatformItf platform = Platform.get();
        Class cl = platform.defineClass(targetClassname, bytecode);
        
        /*
         * Register the class in the repository.
         */
        ClassDefinition cd = new ClassDefinition(cl,bytecode);
        classes.put(targetClassname,cd);
        
        /*
         * If applies, dump the class to the filesystem cache.
         */
        String cachedir = System.getProperty(Juliak.DUMP_GENERATED_PROP_NAME);
        if( cachedir!=null && cachedir.length()!=0 ) {
            try {
                platform.dumpClassToFile( cachedir, targetClassname, bytecode );
            }
            catch(IOException ioe) {
                final String msg = "IOException: " + ioe.getMessage();
                throw new RuntimeException(msg);
            }
        }
        
        return cd;
    }
    
    /**
     * Dump registered classes.
     */
    public void dumpClasses() {
        
        String dirname = System.getProperty("java.io.tmpdir");
        
        for (Iterator iter = classes.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            String clname = (String) entry.getKey();
            ClassDefinition cd = (ClassDefinition) entry.getValue();
            
            try {
                Platform.get().dumpClassToFile(dirname, clname, cd.bytecode );
            }
            catch( IOException ioe ) {
                throw new RuntimeException("IOException: "+ ioe.getMessage());
            }
        }
    }

}
